"""
ya_gpg: Yet Another GPG Module
"""
__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'


# ya_gpg: Yet Another GPG Module ...
# 
# I've done a terrible thing, adding yet another gpg module to yet another
# Zope/Plone product. My only excuses are that 1) I didn't want to
# unnecessarily add a dependency to another product; and 2) I needed some
# functionality I didn't find in other products.
# 
# IMHO, we need a robust product that provides encryption
# services as a tool.
# 
# This module is based on PloneFormMailer's gpg.py, which in turn
# was based on the python-gpg project (since abandoned).
# 
# The PloneFormMailer gpg.py module's details are:
# # Author:       Jens Klein <jens.klein@jensquadrat.com>
# # copied:       parts from python-gpg project (unfinished)
# #
# # Copyright:    (c) 2004 by jens quadrat, Klein & Partner KEG, Austria
# # Licence:      GNU General Public Licence (GPL) Version 2 or later
# 
#
# Changes by SteveM:
#
#  - add --no-secmem-warning flag
#  - don't fail silently.
#    raise exceptions for failures
#  - don't use --status-fd mode, so errors are easier to detect;
#    module wasn't doing anything with them anyway.
#  - if in windows, look for gpg.exe.
#  - add module global gpg object, mainly to avoid
#    repeated path searches, and make it easy to detect
#    when gpg binary is available.

import os, popen2

class GPGError(ValueError):
    " Error from GPG "

class gpg_subprocess:

    # Default path used for searching for the GPG binary, when the
    # PATH environment variable isn't set.
    DEFAULT_PATH = ['/bin', '/usr/bin', '/usr/local/bin']

    def __init__(self, gpg_binary = None):
        """Initialize an object instance.  Options are:

        gpg_binary -- full pathname for GPG binary.  If not supplied,
        the current value of PATH will be searched, falling back to the
        DEFAULT_PATH class variable if PATH isn't available.
        """
        # If needed, look for the gpg binary along the path
        if gpg_binary is None:
            if os.name == 'nt':
                bin_name = 'gpg.exe'
            else:
                bin_name = 'gpg'
            gpg_binary = self._findbinary(bin_name)
        self.gpg_binary = gpg_binary
        if gpg_binary is None:
            raise IOError, "Unable to find gpg binary"

    def _findbinary(self,binname):
        import os
        if os.environ.has_key('PATH'):
            path = os.environ['PATH']
            path = path.split(os.pathsep)
        else:
            path = self.DEFAULT_PATH
        for dir in path:
            fullname = os.path.join(dir, binname)
            if os.path.exists( fullname ):
                return fullname
        return None

    def _open_subprocess(self, *args):
        # Internal method: open a pipe to a GPG subprocess and return
        # the file objects for communicating with it.
        # cmd = self.gpg_binary + ' --yes --status-fd 2 ' + string.join(args)
        cmd = self.gpg_binary + ' --batch --yes --no-secmem-warning ' + " ".join(args)
        child_stdout, child_stdin, child_stderr = popen2.popen3(cmd)
        return child_stdout, child_stdin, child_stderr


    def encrypt(self, data, recipient_key_id):
        "Encrypt the message contained in the string 'data'"

        # get communication objects
        child_stdout, child_stdin, child_stderr = \
            self._open_subprocess('--encrypt',
                '-a',
                '-r '+ recipient_key_id)

        # feed data
        child_stdin.write(data)
        child_stdin.close()
        
        error = child_stderr.read()
        if error:
            raise GPGError, error.replace('\n', '; ')

        # get response
        datagpg = child_stdout.read()

        return datagpg


try:
    gpg = gpg_subprocess()
except IOError:
    gpg = None


if __name__ == '__main__':
    if gpg:
        data = gpg.encrypt('BLABLA','steve')
        print data
    else:
        print "gpg not available"




# Import the base test case classes
from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase

# Make ZopeTestCase aware of the standard products

# These install (or fail) quietly
#ZopeTestCase.installProduct('CMFCore', quiet=1)
#ZopeTestCase.installProduct('CMFDefault', quiet=1)
#ZopeTestCase.installProduct('CMFCalendar', quiet=1)
#ZopeTestCase.installProduct('CMFTopic', quiet=1)
#ZopeTestCase.installProduct('DCWorkflow', quiet=1)
#ZopeTestCase.installProduct('CMFHelpIcons', quiet=1)
#ZopeTestCase.installProduct('CMFQuickInstallerTool', quiet=1)
#ZopeTestCase.installProduct('CMFFormController', quiet=1)
#ZopeTestCase.installProduct('GroupUserFolder', quiet=1)
#ZopeTestCase.installProduct('ZCTextIndex', quiet=1)
#ZopeTestCase.installProduct('SecureMailHost', quiet=1)
#ZopeTestCase.installProduct('CMFPlone')
#ZopeTestCase.installProduct('Archetypes')
#ZopeTestCase.installProduct('PortalTransforms', quiet=1)
#ZopeTestCase.installProduct('MimetypesRegistry', quiet=1)
#ZopeTestCase.installProduct('kupu', quiet=1)

# These must install cleanly
ZopeTestCase.installProduct('PloneFormGen')

# Set up the Plone site used for the test fixture. The PRODUCTS are the products
# to install in the Plone site (as opposed to the products defined above, which
# are all products available to Zope in the test fixture)
PRODUCTS = ['PloneFormGen']
PloneTestCase.setupPloneSite(products=PRODUCTS)


class PloneFormGenTestCase(PloneTestCase.PloneTestCase):

    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def _setup(self):
        PloneTestCase.PloneTestCase._setup(self)
        self.app.REQUEST['SESSION'] = self.Session()



"""DataGridFF install routine"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'


from Products.CMFCore.utils import getToolByName

from Products.CMFPlone.utils import safe_hasattr

from Products.CMFFormController.FormAction import FormActionKey

from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
from Products.Archetypes.public import listTypes

from StringIO import StringIO

from Products.PFGDataGrid.config import *

def install(self):
    """Install PFGDataGrid"""

    
    #######################
    # Both PloneFormGen and the target field provider are going to have to be installed
    # first. So, let's check.
    
    portal_skins = getToolByName(self, 'portal_skins')
    assert safe_hasattr(portal_skins, 'DataGridWidget'), "DataGridField must be installed prior to installing this product."
    assert safe_hasattr(portal_skins, 'PloneFormGen'), "PloneFormGen must be installed prior to installing this product."

    # Safe to proceed.

    classes = listTypes(PROJECTNAME)
    
    # get a list of types provided by this product
    # note that this trick will cause problems if the meta_type doesn't match class name
    myTypes = [item['name'] for item in classes]

    out = StringIO()

    print >> out, "Installing %s" % PROJECTNAME


    # boilerplate setup of DynamicViewFTI and factory

    installTypes(self, out,
                 classes,
                 PROJECTNAME)
    print >> out, "Installed types"

    factory = getToolByName(self, 'portal_factory')
    types = factory.getFactoryTypes().keys()
    for add_type in myTypes:
        if add_type not in types:
            types.append(add_type)
            factory.manage_setPortalFactoryTypes(listOfTypeIds = types)

    print >> out, "Added %s to portal_factory" % ', '.join(myTypes)


    #######################
    # form fields should not be visible to navigation or search.
    # they should have no workflow of their own.
    
    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')

    # Add to types_not_searched
    typesNotSearched = list(siteProperties.getProperty('types_not_searched'))
    for f in myTypes:
        if f not in typesNotSearched:
            typesNotSearched.append(f)
    siteProperties.manage_changeProperties(types_not_searched = typesNotSearched)
    print >> out, "Added %ss to types_not_searched" % ', '.join(myTypes)

    # Add to types excluded from navigation
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in myTypes:
        if f not in typesNotListed:
            typesNotListed.append(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Added %s to metaTypesNotToList" % ', '.join(myTypes)

    # Set up the workflow: there should be none!
    wft = getToolByName(self, 'portal_workflow')
    wft.setChainForPortalTypes(myTypes, ())
    print >> out, "Set up empty workflows for %s." % ', '.join(myTypes)


    #######################
    # Here's the code specific to making this visible to PloneFormGen

    portal_types = getToolByName(self, 'portal_types')
    for typeName in ('FormFolder', 'FieldsetFolder'):
        ptType = portal_types.getTypeInfo(typeName)
        ffact = list(ptType.allowed_content_types)
        ffact += myTypes
        ptType.manage_changeProperties(allowed_content_types = ffact)
    print >> out, "Added %s to allowed_content_types for %s" % (', '.join(myTypes), typeName)

    # End PFG installation
    #######################
    


    print >> out, "Done."
    return out.getvalue()






#
# Test PloneFormGen top-level functionality
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import zope

from Products.PloneFormGen.tests import pfgtc

from Products.CMFCore.utils import getToolByName

class FakeRequest(dict):
    
    def __init__(self, **kwargs):
        self.form = kwargs


class TestTools(pfgtc.PloneFormGenTestCase):
    """ test our tool """

    def test_FormGenTool(self):
        fgt = getToolByName(self.portal, 'formgen_tool')

        pt = getToolByName(self.portal, 'portal_properties').ploneformgen_properties

        fgt.setDefault('permissions_used', ['test text'])
        fgt.setDefault('mail_template', 'something')
        fgt.setDefault('mail_body_type', 'text')
        fgt.setDefault('mail_recipient_email', 'eggs')
        fgt.setDefault('mail_recipient_name', 'spam')
        fgt.setDefault('mail_cc_recipients', ['spam and eggs'])
        fgt.setDefault('mail_bcc_recipients', ['eggs and spam'])
        fgt.setDefault('mail_xinfo_headers', ['one', 'two'])
        fgt.setDefault('mail_add_headers',['three', 'four'])
        
        permits = fgt.getPfgPermissions()
        self.failUnlessEqual(len(permits), 1)
        self.failUnlessEqual(permits[0], 'test text')
        
        self.failUnlessEqual(fgt.getDefaultMailTemplateBody(), 'something')
        self.failUnlessEqual(fgt.getDefaultMailBodyType(), 'text')
        self.failUnlessEqual(fgt.getDefaultMailRecipient(), 'eggs')        
        self.failUnlessEqual(fgt.getDefaultMailRecipientName(), 'spam')

        cc = fgt.getDefaultMailCC()
        self.failUnlessEqual(len(cc), 1)
        self.failUnlessEqual(cc[0], 'spam and eggs')
        
        bcc = fgt.getDefaultMailBCC()
        self.failUnlessEqual(len(bcc), 1)
        self.failUnlessEqual(bcc[0], 'eggs and spam')
        
        xi = fgt.getDefaultMailXInfo()
        self.failUnlessEqual(len(xi), 2)
        self.failUnlessEqual(xi[0], 'one')
        
        xi = fgt.getDefaultMailAddHdrs()
        self.failUnlessEqual(len(xi), 2)
        self.failUnlessEqual(xi[0], 'three')
        
        
    def test_toolRolesForPermission(self):
        fgt = getToolByName(self.portal, 'formgen_tool')

        # make sure rolesForPermission works
        roleList = fgt.rolesForPermission('PloneFormGen: Add Content')
        self.failIfEqual(len(roleList), 0)
        mid = ''
        oid = ''
        for role in roleList:
            if role['label'] == 'Manager':
                self.failUnlessEqual(role['checked'], 'CHECKED')
                mid = role['id']
            if role['label'] == 'Owner':
                self.failUnlessEqual(role['checked'], 'CHECKED')
                oid = role['id']
        self.failUnless( mid )
        self.failUnless( oid )

        # let's try changing a permission

        # first, get the request ids
        roleList = fgt.rolesForPermission('PloneFormGen: Edit Advanced Fields')
        self.failIfEqual(len(roleList), 0)
        mid = ''
        oid = ''
        for role in roleList:
            if role['label'] == 'Manager':
                self.failUnlessEqual(role['checked'], 'CHECKED')
                mid = role['id']
            if role['label'] == 'Owner':
                self.failUnlessEqual(role['checked'], None)
                oid = role['id']
        self.failUnless( mid )
        self.failUnless( oid )
        
        fr = FakeRequest()
        fr.form[mid] = '1'
        fr.form[oid] = '1'
        fr.form['PloneFormGen: Edit Advanced Fields'] = '1'
        fgt.setRolePermits(fr)
        
        # now, check to see if it took
        roleList = fgt.rolesForPermission('PloneFormGen: Edit Advanced Fields')
        self.failIfEqual(len(roleList), 0)
        mid = ''
        oid = ''
        for role in roleList:
            if role['label'] == 'Manager':
                self.failUnlessEqual(role['checked'], 'CHECKED')
                mid = role['id']
            if role['label'] == 'Owner':
                self.failUnlessEqual(role['checked'], 'CHECKED')
                oid = role['id']
        self.failUnless( mid )
        self.failUnless( oid )



if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestTools))
    return suite



from zope.interface import Interface


class IPloneFormGenField(Interface):
    """fgField marker interface
    """

    # fgField -- an archetypes field; must exist as an object (not class) attribute
    
    def fgPrimeDefaults(request, useTALES=True):
        """ primes request with default """

    def fgFields(request=None):
        """ generate fields on the fly; also primes request with defaults """

    def fgvalidate(REQUEST=None, errors=None, data=None, metadata=None):
        """Validates the field data from the request.
        """

    def isLabel():
        """Returns True if this field is a label and should only be
           shown in the edit form.
        """

    def isFileField():
        """Returns True if the embedded field acts like a file field
        """
        
    def thanksValue(REQUEST):
        """ return from REQUEST, this field's value, rendered for display
            on a thanks page.
        """

    def getFieldFormName():
        """Returns field name as used in form and request.form.
           To be overridden when form field name varies from the field name,
           as, for example, with file field."""



"""actionAdapter -- A base adapter for form actions"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

from zope.interface import implements

from Products.PloneFormGen.config import *

from AccessControl import ClassSecurityInfo

from Products.Archetypes.public import *
from Products.Archetypes.utils import shasattr

from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content.base import registerATCT
from Products.CMFCore.permissions import View, ModifyPortalContent

from Products.TALESField import TALESString

from Products.PloneFormGen import HAS_PLONE30
from Products.PloneFormGen.interfaces import IPloneFormGenActionAdapter

FormAdapterSchema = ATContentTypeSchema.copy() + Schema((
    TALESString('execCondition',
        schemata='overrides',
        searchable=0,
        required=0,
        validators=('talesvalidator',),
        default='',
        write_permission=EDIT_TALES_PERMISSION,
        read_permission=ModifyPortalContent,
        isMetadata=True, # just to hide from base view
        widget=StringWidget(label="Execution Condition",
            description="""
                A TALES expression that will be evaluated to determine whether or not
                to execute this action.
                Leave empty if unneeded, and the ection will be executed. 
                Your expression should evaluate as a boolean; return True if you wish
                the action to execute.
                PLEASE NOTE: errors in the evaluation of this expression will cause
                an error on form display.
            """,
            size=70,
            i18n_domain = "ploneformgen",
            label_msgid = "label_execcondition_text",
            description_msgid = "help_execcondition_text",
        ),
    ),
    ))

if not HAS_PLONE30:
    FormAdapterSchema['description'].schemata = 'metadata'
finalizeATCTSchema(FormAdapterSchema, folderish=True, moveDiscussion=False)

if HAS_PLONE30:
    # avoid showing unnecessary schema tabs
    for afield in ('description',
                   'subject', 
                   'relatedItems', 
                   'location', 
                   'language', 
                   'effectiveDate', 
                   'expirationDate', 
                   'creation_date', 
                   'modification_date', 
                   'creators', 
                   'contributors', 
                   'rights', 
                   'allowDiscussion', 
                   'excludeFromNav', ):
        FormAdapterSchema[afield].widget.visible = {'view':'invisible','edit':'invisible'}
        FormAdapterSchema[afield].schemata = 'default'


class FormActionAdapter(ATCTContent):
    """A base action adapter"""

    implements(IPloneFormGenActionAdapter)

    schema         =  FormAdapterSchema

    content_icon   = 'FormAction.gif'
    meta_type      = 'FormActionAdapter'
    portal_type    = 'FormActionAdapter'
    archetype_name = 'Form Action Adapter'

    immediate_view = 'base_view'
    default_view   = 'base_view'
    suppl_views = ()

    typeDescription= 'An adapter that supplies a form action.'

    global_allow = 0    

    security       = ClassSecurityInfo()


    def onSuccess(self, fields, REQUEST=None):
        """ Called by form to invoke custom success processing.
            return None (or don't use "return" at all) if processing is
            error-free.
            
            Return a dictionary like {'field_id':'Error Message'}
            and PFG will stop processing action adapters and
            return back to the form to display your error messages
            for the matching field(s).

            You may also use Products.PloneFormGen.config.FORM_ERROR_MARKER
            as a marker for a message to replace the top-of-the-form error
            message.

            For example, to set a message for the whole form, but not an
            individual field:

            {FORM_ERROR_MARKER:'Yuck! You will need to submit again.'}

            For both a field and form error:

            {FORM_ERROR_MARKER:'Yuck! You will need to submit again.',
             'field_id':'Error Message for field.'}
            
            Messages may be string types or zope.i18nmessageid objects.                
        """
        
        # fields will be a sequence of objects with an IPloneFormGenField interface
        
        pass


    def at_post_create_script(self):
        """ activate action adapter in parent folder """
        
        # XXX TODO - change to use events when we give up on Plone 2.1.x
        
        ATCTContent.at_post_create_script(self)

        self.aq_parent.addActionAdapter(self.id)
        



"""DataGridFF -- a PloneFormGen form field built on DataGridField"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

import cgi

from AccessControl import ClassSecurityInfo

from Products.CMFCore.permissions import View, ModifyPortalContent

from Products.Archetypes.public import *

from Products.ATContentTypes.content.base import ATCTContent, ATContentTypeSchema
from Products.ATContentTypes.content.folder import finalizeATCTSchema

from Products.PFGDataGrid.config import *

from Products.PloneFormGen.content import fieldsBase

from Products.DataGridField import \
    DataGridField, DataGridWidget, Column


class FormDataGridField(fieldsBase.BaseFormField):
    """
        A PloneFormGen Form Field
    """

    security  = ClassSecurityInfo()


    #######################
    # We could derive our schema from several schema defined in
    # fieldsBase. In general, choose by which has a matching data
    # type for the default field value.
    #
    # DataGridField won't match any, so we'll use the BaseFieldSchema,
    # which has no default, and provide a getFgDefault method to make
    # up for it.

    schema = fieldsBase.BaseFieldSchema.copy() + Schema((

        # We'll make DataGridField eat its own dog food by using it
        # to get our column definitions and defaults.
        DataGridField('columnDefs',
            searchable = False,
            required = True,
            columns=('columnId','columnTitle','columnDefault'),                
            default = [ {'columnId':'column1', 'columnTitle':'Column One', 'columnDefault':''}, ],
            widget = DataGridWidget(
                label = 'Column Definitions',
                i18n_domain = "pfgdatagrid",
                label_msgid = "label_column_defs_text",
                description = """
                    Specify a unique identifier and a title for each column
                    you wish to appear in the datagrid. The default value
                    is optional.
                """,
                description_msgid = "help_column_defs_text",
                columns={
                    'columnId':Column('Column Id'),
                    'columnTitle':Column('Column Title'),
                    'columnDefault':Column('Default Value'),
                },
            ),
        ),
        # A few DFG-specific fields. We'll need to provide
        # mutator and accessor methods that tie these
        # to values in the widget
        BooleanField('allowDelete',
            searchable = False,
            default = '1',
            widget = BooleanWidget(
                label = 'Allow Row Deletion',
                i18n_domain = "pfgdatagrid",
                label_msgid = "label_allow_delete_text",
            ),
        ),
        BooleanField('allowInsert',
            searchable = False,
            default = '1',
            widget = BooleanWidget(
                label = 'Allow Row Insertion',
                i18n_domain = "pfgdatagrid",
                label_msgid = "label_allow_insert_text",
            ),
        ),
        BooleanField('allowReorder',
            searchable = False,
            default = '1',
            widget = BooleanWidget(
                label = 'Allow Row Reordering',
                i18n_domain = "pfgdatagrid",
                label_msgid = "label_allow_reorder_text",
            ),
        ),

    ))
    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)

    content_icon   = 'BasicField.gif'

    meta_type      = \
    portal_type    = 'FormDataGridField'
    archetype_name = 'DataGrid Field'

    typeDescription= 'A flexible datagrid field.'
    typeDescMsgId  = 'datagridformfield_description'


    #######################
    # Every form field must have an fgField attribute that is a field
    # definition created outside of the normal schema.
    # Since this needs to be an instance attribute, we'll set it up
    # in the __init__ method.
    
    def __init__(self, oid, **kwargs):
        """ initialize class """

        fieldsBase.BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = DataGridField('fg_datagrid_field',
            searchable=False,
            required=False,
            write_permission = View,
            widget = DataGridWidget(),
            columns=('column1','column2','The third'),                
            allow_delete = True,
            allow_insert = True,
            allow_reorder = True,
            )



    #######################
    # Mutators and accessors that tie items in the form field's
    # schema to attributes of the fgField field.
    # Most are simple.
    # setColumnDefs is more complex because it's translating the
    # data from the columnDefs DataGrid for multiple uses.


    security.declareProtected(ModifyPortalContent, 'setColumnDefs')
    def setColumnDefs(self, value, **kwa):
        """ mutator for columnDefs """

        myval = [col for col in value if col.get('columnId')]        
        self.columnDefs = myval
        self.fgField.columns = [col['columnId'] for col in myval]
        
        res = {}
        for col in myval:
            res[ col['columnId'] ] = Column( col['columnTitle'], default=col['columnDefault'] )
        self.fgField.widget.columns = res
        

    security.declareProtected(ModifyPortalContent, 'setAllowDelete')
    def setAllowDelete(self, value, **kwa):
        """ set allow_delete flag for field """

        # Note: booleans come in the request as '0' or '1';
        # we need to translate them into True or False.
        self.fgField.allow_delete = value == '1'


    security.declareProtected(View, 'getAllowDelete')
    def getAllowDelete(self, **kw):
        """ get allow_delete flag for field """

        return self.fgField.allow_delete


    security.declareProtected(ModifyPortalContent, 'setAllowInsert')
    def setAllowInsert(self, value, **kwa):
        """ set allow_insert flag for field """

        self.fgField.allow_insert = value == '1'


    security.declareProtected(View, 'getAllowInsert')
    def getAllowInsert(self, **kw):
        """ get allow_insert flag for field """

        return self.fgField.allow_insert


    security.declareProtected(ModifyPortalContent, 'setAllowReorder')
    def setAllowReorder(self, value, **kwa):
        """ set allow_reorder flag for field """

        self.fgField.allow_reorder = value == '1'


    security.declareProtected(View, 'getAllowReorder')
    def getAllowReorder(self, **kw):
        """ get allow_reorder flag for field """

        return self.fgField.allow_reorder


    #######################
    # Due to the very-specific nature of the DataGridField
    # default, we're going to provide it via an accessor method.
    # If the data structure was common, we could have done it in
    # the form field schema with an fgDefault field.

    def getFgDefault(self, **kwargs):
        """
         supply defaults for fgField datagrid as a list
         with a single dictionary 
        """

        res = {}
        for col in self.columnDefs:
            if col.get('columnId'):
                res[col['columnId']] = col['columnDefault']
        return [res]
    

    #######################
    # The string representation of the DataGridField value isn't suitable
    # for public consumption. So, let's massage it by overriding
    # the htmlValue method

    def htmlValue(self, REQUEST):
        """ return from REQUEST, this field's value in html.
        """

        # get column titles
        titles = {}
        for col in self.columnDefs:
            titles[col['columnId']] = col['columnTitle']

        value = REQUEST.form.get(self.__name__, 'No Input')
        
        res = "<dl>"
        for adict in value:
            if adict.get('orderindex_', '') != 'template_row_marker':
                for akey in adict.keys():
                    if akey != 'orderindex_':
                        res = "%s\n<dt>%s</dt>\n<dd>%s<dd>" % \
                         (res, cgi.escape(titles[akey]), cgi.escape(adict[akey]))
        return "%s</dl>" % res
    

    
registerType(FormDataGridField, PROJECTNAME)

